# -*- coding: utf-8 -*-
"""
filometro.filometro
-------------------

Fornece a API pública principal do pacote.
"""

__all__ = ['Filometro']

from filometro import __version__
from filometro import __author__

from typing import List

from filometro.deolhonafila import APIDeOlhoNaFila

from filometro.dataclasses import Posto

from filometro import convert

from filometro.enums import Zone
from filometro.enums import Modality
from filometro.enums import Situation
from filometro.enums import Immunizing
from filometro.enums import District


class Filometro():
    """Filometro é a API príncipal do pacote. 
    
    Fornence os métodos para coletar e filtrar os dados dos postos.
    """

    def __init__(self) -> None:
        self._api = APIDeOlhoNaFila()

        self._postos = self._load_postos()
    
    def _load_postos(self) -> List[Posto]:
        postos_dicts = self._api.get_data()
        postos_objects = convert.postos_dicts_to_postos_objects(postos_dicts)
        return postos_objects

    def reload(self) -> None:
        """Recarregar os dados com as informações mais recentes."""

        postos = self._load_postos()
        self._postos = postos

    def all_postos(self) -> List[Posto]:
        """Retorna os dados de todos os postos."""

        return self._postos

    def by_zone(self, zone: Zone) -> List[Posto]:
        """Retorna os dados dos postos por zona selecionada."""

        return [posto for posto in self._postos if posto.zone == zone.value]

    def by_modality(self, modality: Modality) -> List[Posto]:
        """Retorna os dados dos postos por modalidade selecionada."""

        return [posto for posto in self._postos if posto.modality == modality.value]

    def by_district(self, district: District) -> List[Posto]:
        """Retorna os dados dos postos por distrito selecionado."""

        return [posto for posto in self._postos if posto.district == district.value]

    def by_situation(self, situation: Situation) -> List[Posto]:
        """Retorna os dados dos postos por situação selecionada."""

        return [posto for posto in self._postos if posto.situation == situation.value]

    def by_immunizing(self, immunizing: Immunizing) -> List[Posto]:
        """Retorna os dados dos postos por imunizante selecionado."""
        
        return [posto for posto in self._postos if posto.__dict__[immunizing.value] == '1']



# -*- coding: utf-8 -*-
"""
filometro.enums
---------------

Fornece todas as enumerações constantes para facilitar a buscas dos dados.
"""

__all__ = [
    'Zone',
    'Modality',
    'Situation',
    'Immunizing',
    'District'
]

from filometro import __version__
from filometro import __author__

from enum import Enum


class Zone(Enum):
    SUL = 'SUL'
    OESTE = 'OESTE'
    NORTE = 'NORTE'
    LESTE = 'LESTE'
    CENTRO = 'CENTRO'
    MEGA_DRIVES = 'MEGA-DRIVES'


class Modality(Enum):
    PARQUES = 'PARQUES'
    POSTO_FIXO = 'POSTO FIXO'
    POSTO_VOLANTE = 'POSTO VOLANTE'
    DRIVE_THRU = 'DRIVE-THRU'
    MEGAPOSTO = 'MEGAPOSTO'


class Situation(Enum):
    NAO_FUNCIONANDO = 'NÃO FUNCIONANDO'
    SEM_FILA = 'SEM FILA'
    FILA_PEQUENA = 'FILA PEQUENA'
    FILA_MEDIA = 'FILA MÉDIA'
    FILA_GRANDE = 'FILA GRANDE'


class Immunizing(Enum):
    ASTRAZENECA = 'astrazeneca'
    INTERCAMBIALIDADE = 'intercambialidade'
    PFIZER = 'pfizer'
    CORONAVAC = 'coronavac'
    JANSSEN = 'janssen'


class District(Enum):
    AGUA_RASA = 'Água Rasa'
    ALTO_DE_PINHEIROS = 'Alto de Pinheiros'
    ANHANGUERA = 'Anhanguera'
    ARICANDUVA = 'Aricanduva'
    ARTUR_ALVIM = 'Artur Alvim'
    BELA_VISTA = 'Bela Vista'  
    BELEM = 'Belém'
    BOM_RETIRO = 'Bom Retiro'  
    BRASILANDIA = 'Brasilândia'
    BRAS = 'Brás'
    BUTANTA = 'Butantã'
    CACHOEIRINHA = 'Cachoeirinha'
    CAMBUCI = 'Cambuci'
    CAMPO_BELO = 'Campo Belo'
    CAMPO_GRANDE = 'Campo Grande'
    CAMPO_LIMPO = 'Campo Limpo'
    CANGAIBA = 'Cangaíba'
    CAPAO_REDONDO = 'Capão Redondo'
    CARRAO = 'Carrão'
    CASA_VERDE = 'Casa Verde'
    CIDADE_ADEMAR = 'Cidade Ademar'
    CIDADE_DUTRA = 'Cidade Dutra'
    CIDADE_LIDER = 'Cidade Líder'
    CIDADE_TIRADENTES = 'Cidade Tiradentes'
    CURSINO = 'Cursino'
    ERMELINO_MATARAZZO = 'Ermelino Matarazzo'
    FREGUESIA_DO_O = 'Freguesia do Ó'
    GRAJAU = 'Grajaú'
    GUAIANASES = 'Guaianases'
    IGUATEMI = 'Iguatemi'
    IPIRANGA = 'Ipiranga'
    ITAIM_BIBI = 'Itaim Bibi'
    ITAIM_PAULISTA = 'Itaim Paulista'
    ITAQUERA = 'Itaquera'
    JABAQUARA = 'Jabaquara'
    JAGUARA = 'Jaguara'
    JAGUARE = 'Jaguaré'
    JARAGUA = 'Jaraguá'
    JARDIM_HELENA = 'Jardim Helena'
    JARDIM_SAO_LUIS = 'Jardim São Luis'
    JARDIM_ANGELA = 'Jardim Ângela'
    JACANA = 'Jaçanã'
    JOSE_BONIFACIO = 'José Bonifácio'
    LAJEADO = 'Lajeado'
    LAPA = 'Lapa'
    LIMAO = 'Limão'
    MANDAQUI = 'Mandaqui'
    MARSILAC = 'Marsilac'
    MOEMA = 'Moema'
    MOOCA = 'Mooca'
    MORUMBI = 'Morumbi'
    PARELHEIROS = 'Parelheiros'
    PARI = 'Pari'
    PARQUE_DO_CARMO = 'Parque do Carmo'
    PEDREIRA = 'Pedreira'
    PENHA = 'Penha'
    PERDIZES = 'Perdizes'
    PERUS = 'Perus'
    PINHEIROS = 'Pinheiros'
    PIRITUBA = 'Pirituba'
    PONTE_RASA = 'Ponte Rasa'
    RAPOSO_TAVARES = 'Raposo Tavares'
    RIO_PEQUENO = 'Rio Pequeno'
    SACOMA = 'Sacomã'
    SANTA_CECILIA = 'Santa Cecília'
    SANTANA = 'Santana'
    SANTO_AMARO = 'Santo Amaro'
    SAPOPEMBA = 'Sapopemba'
    SAUDE = 'Saúde'
    SOCORRO = 'Socorro'
    SAO_DOMINGOS = 'São Domingos'
    SAO_LUCAS = 'São Lucas'
    SAO_MATEUS = 'São Mateus'
    SAO_MIGUEL = 'São Miguel'
    SAO_RAFAEL = 'São Rafael'
    SE = 'Sé'
    TATUAPE = 'Tatuapé'
    TREMEMBE = 'Tremembé'
    TUCURUVI = 'Tucuruvi'
    VILA_ANDRADE = 'Vila Andrade'
    VILA_CURUCA = 'Vila Curuçá'
    VILA_FORMOSA = 'Vila Formosa'
    VILA_GUILHERME = 'Vila Guilherme'
    VILA_JACUI = 'Vila Jacuí'
    VILA_LEOPOLDINA = 'Vila Leopoldina'
    VILA_MARIA = 'Vila Maria'
    VILA_MARIANA = 'Vila Mariana'
    VILA_MATILDE = 'Vila Matilde'
    VILA_MEDEIROS = 'Vila Medeiros'
    VILA_PRUDENTE = 'Vila Prudente'
    VILA_SONIA = 'Vila Sônia'



# -*- coding: utf-8 -*-
"""
filometro.convert
-----------------

Fornece as funções de conversão de dados utilizada no pacote.
"""

__all__ = [
    'posto_dict_to_posto_object',
    'postos_dicts_to_postos_objects'
]

from filometro import __version__
from filometro import __author__

from typing import List

from filometro.dataclasses import Posto


def posto_dict_to_posto_object(posto_dict: dict) -> Posto:
    return Posto(
        equipment=posto_dict['equipamento'],
        address=posto_dict['endereco'],
        district=posto_dict['distrito'],
        zone=posto_dict['crs'],
        astrazeneca=posto_dict['astrazeneca'],
        coronavac=posto_dict['coronavac'],
        pfizer=posto_dict['pfizer'],
        janssen=posto_dict['janssen'],
        intercambialidade=posto_dict['intercambialidade'],
        situation=posto_dict['status_fila'],
        modality=posto_dict['tipo_posto'],
        last_update=posto_dict['data_hora'],
        _index_situation=posto_dict['indice_fila'],
        _id_district=posto_dict['id_distrito'],
        _id_zone=posto_dict['id_crs'],
        _id_tb_unidades=posto_dict['id_tb_unidades'],
        _id_modality=posto_dict['id_tipo_posto']
    )


def postos_dicts_to_postos_objects(postos_dicts: List[dict]) -> List[Posto]:
    postos_objects = []
    for posto_dict in postos_dicts:
        posto_object = posto_dict_to_posto_object(posto_dict)
        postos_objects.append(posto_object)
    
    return postos_objects



# -*- coding: utf-8 -*-
"""
filometro.deolhonafila
----------------------

Disponíbiliza formas de ter acesso aos dados do site 'De Olho na Fila'.
"""

__all__ = ['APIDeOlhoNaFila']

from filometro import __version__
from filometro import __author__

import requests

from typing import List


class APIDeOlhoNaFila():
    """Um wrapper da API do site 'De Olho na Fila'."""

    def __init__(self) -> None:
        self.endpoint = 'https://deolhonafila.prefeitura.sp.gov.br/processadores/dados.php'
        self.payload = {'dados': 'dados'}
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/95.0.4638.69 Safari/537.36',
            'Host': 'deolhonafila.prefeitura.sp.gov.br',
            'Connection': 'keep-alive',
            'Content-Length': '11',
            'sec-ch-ua': '"Google Chrome";v="95", "Chromium";v="95", ";Not A Brand";v="99"',
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'X-Requested-With': 'XMLHttpRequest',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
            'Origin': 'https://deolhonafila.prefeitura.sp.gov.br',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': 'https://deolhonafila.prefeitura.sp.gov.br/',
            'Accept-Encoding': 'gzip, deflate, br',
            'Accept-Language': 'pt,en-US;q=0.9,en;q=0.8,de;q=0.7',
            'dnt': '1',
            'sec-gpc': '1'
        }

    def get_data(self) -> List[dict]:
        """Pegue todos os dados do site 'De Olho na Fila'."""

        response = requests.post(self.endpoint, headers=self.headers, data=self.payload)

        response.raise_for_status()

        return response.json()



# -*- coding: utf-8 -*-
"""
filometro.dataclasses
---------------------

Fornece os modelos de dados utilizados pelo pacote.
"""

__all__ = ['Posto']

from filometro import __version__
from filometro import __author__

from dataclasses import dataclass
from dataclasses import field


@dataclass
class Posto():
    equipment: str                               # equipamento
    address: str = field(repr=False)             # endereço
    district: str = field(repr=False)            # distrito
    zone: str = field(repr=False)                # crs (Coordinate Reference Systems)
    astrazeneca: str = field(repr=False)
    coronavac: str = field(repr=False)
    pfizer: str = field(repr=False)
    janssen: str = field(repr=False)
    intercambialidade: str = field(repr=False)
    situation: str = field(repr=False)           # status_fila
    modality: str = field(repr=False)            # tipo_posto
    last_update: str                             # data_hora
    _index_situation: str = field(repr=False)    # indice_fila
    _id_district: str = field(repr=False)        # id_distrito
    _id_zone: str = field(repr=False)            # id_crs
    _id_tb_unidades: str = field(repr=False)
    _id_modality: str = field(repr=False)        # id_tipo_posto
